@@ -30,25 +30,22 @@ import pkgutil
 import re
 import six
 
-from keystoneclient import access
-from keystoneclient import adapter
-from keystoneclient.auth.identity import base
-from keystoneclient import discover
+from keystoneauth1 import access
+from keystoneauth1 import adapter
+from keystoneauth1 import discover
+from keystoneauth1.identity import base
+from oslo_utils import encodeutils
+from oslo_utils import importutils
+from oslo_utils import strutils
+osprofiler_web = importutils.try_import("osprofiler.web")  # noqa
 import requests
+from six.moves import urllib
+import six.moves.urllib.parse as urlparse
 
+from cinderclient._i18n import _
+from cinderclient import api_versions
 from cinderclient import exceptions
 import cinderclient.extension
-from cinderclient.openstack.common import importutils
-from cinderclient.openstack.common.gettextutils import _
-from oslo_utils import encodeutils
-from oslo_utils import strutils
-
-osprofiler_web = importutils.try_import("osprofiler.web")
-
-try:
-    import urlparse
-except ImportError:
-    import urllib.parse as urlparse
 
 try:
     from eventlet import sleep
@@ -60,18 +57,75 @@ try:
 except ImportError:
     import simplejson as json
 
-# Python 2.5 compat fix
-if not hasattr(urlparse, 'parse_qsl'):
-    import cgi
-    urlparse.parse_qsl = cgi.parse_qsl
-
-_VALID_VERSIONS = ['v1', 'v2']
-
+_VALID_VERSIONS = ['v1', 'v2', 'v3']
+V3_SERVICE_TYPE = 'volumev3'
+V2_SERVICE_TYPE = 'volumev2'
+V1_SERVICE_TYPE = 'volume'
+SERVICE_TYPES = {'1': V1_SERVICE_TYPE,
+                 '2': V2_SERVICE_TYPE,
+                 '3': V3_SERVICE_TYPE}
+REQ_ID_HEADER = 'X-OpenStack-Request-ID'
 
 # tell keystoneclient that we can ignore the /v1|v2/{project_id} component of
 # the service catalog when doing discovery lookups
-for svc in ('volume', 'volumev2'):
-    discover.add_catalog_discover_hack(svc, re.compile('/v[12]/\w+/?$'), '/')
+for svc in ('volume', 'volumev2', 'volumev3'):
+    discover.add_catalog_discover_hack(svc, re.compile(r'/v[12]/\w+/?$'), '/')
+
+
+def get_server_version(url):
+    """Queries the server via the naked endpoint and gets version info.
+
+    :param url: url of the cinder endpoint
+    :returns: APIVersion object for min and max version supported by
+              the server
+    """
+    min_version = "2.0"
+    current_version = "2.0"
+
+    logger = logging.getLogger(__name__)
+    try:
+        u = urllib.parse.urlparse(url)
+        version_url = None
+
+        # NOTE(andreykurilin): endpoint URL has at least 2 formats:
+        #   1. The classic (legacy) endpoint:
+        #       http://{host}:{optional_port}/v{1 or 2 or 3}/{project-id}
+        #       http://{host}:{optional_port}/v{1 or 2 or 3}
+        #   3. Under wsgi:
+        #       http://{host}:{optional_port}/volume/v{1 or 2 or 3}
+        for ver in ['v1', 'v2', 'v3']:
+            if u.path.endswith(ver) or "/{0}/".format(ver) in u.path:
+                path = u.path[:u.path.rfind(ver)]
+                version_url = '%s://%s%s' % (u.scheme, u.netloc, path)
+                break
+
+        if not version_url:
+            # NOTE(andreykurilin): probably, it is one of the next cases:
+            #  * https://volume.example.com/
+            #  * https://example.com/volume
+            # leave as is without cropping.
+            version_url = url
+
+        response = requests.get(version_url)
+        data = json.loads(response.text)
+        versions = data['versions']
+        for version in versions:
+            if '3.' in version['version']:
+                min_version = version['min_version']
+                current_version = version['version']
+                break
+    except exceptions.ClientException as e:
+        logger.warning("Error in server version query:%s\n"
+                       "Returning APIVersion 2.0", six.text_type(e.message))
+    return (api_versions.APIVersion(min_version),
+            api_versions.APIVersion(current_version))
+
+
+def get_highest_client_server_version(url):
+    """Returns highest supported version by client and server as a string."""
+    min_server, max_server = get_server_version(url)
+    max_client = api_versions.APIVersion(api_versions.MAX_VERSION)
+    return min(max_server, max_client).get_string()
 
 
 def get_volume_api_from_url(url):
@@ -87,25 +141,60 @@ def get_volume_api_from_url(url):
     raise exceptions.UnsupportedVersion(msg)
 
 
+def _log_request_id(logger, resp, service_name):
+    request_id = resp.headers.get('x-openstack-request-id')
+    if request_id:
+        logger.debug('%(method)s call to %(service_type)s for %(url)s '
+                     'used request id %(response_request_id)s',
+                     {'method': resp.request.method,
+                      'service_type': service_name,
+                      'url': resp.url, 'response_request_id': request_id})
+
+
 class SessionClient(adapter.LegacyJsonAdapter):
 
+    def __init__(self, *args, **kwargs):
+        apiver = kwargs.pop('api_version', None) or api_versions.APIVersion()
+        if not isinstance(apiver, api_versions.APIVersion):
+            apiver = api_versions.APIVersion(str(apiver))
+        if apiver.ver_minor != 0:
+            kwargs['default_microversion'] = apiver.get_string()
+        self.retries = kwargs.pop('retries', 0)
+        self._logger = logging.getLogger(__name__)
+        super(SessionClient, self).__init__(*args, **kwargs)
+
     def request(self, *args, **kwargs):
         kwargs.setdefault('authenticated', False)
+
         # Note(tpatil): The standard call raises errors from
-        # keystoneclient, here we need to raise the cinderclient errors.
+        # keystoneauth, here we need to raise the cinderclient errors.
         raise_exc = kwargs.pop('raise_exc', True)
         resp, body = super(SessionClient, self).request(*args,
                                                         raise_exc=False,
                                                         **kwargs)
+
         if raise_exc and resp.status_code >= 400:
             raise exceptions.from_response(resp, body)
 
+        if not self.global_request_id:
+            self.global_request_id = resp.headers.get('x-openstack-request-id')
+
         return resp, body
 
     def _cs_request(self, url, method, **kwargs):
         # this function is mostly redundant but makes compatibility easier
         kwargs.setdefault('authenticated', True)
-        return self.request(url, method, **kwargs)
+        attempts = 0
+        while True:
+            attempts += 1
+            try:
+                return self.request(url, method, **kwargs)
+            except exceptions.OverLimit as overlim:
+                if attempts > self.retries or overlim.retry_after < 1:
+                    raise
+                msg = "Retrying after %s seconds." % overlim.retry_after
+                self._logger.debug(msg)
+                sleep(overlim.retry_after)
 
     def get(self, url, **kwargs):
         return self._cs_request(url, 'GET', **kwargs)
@@ -119,12 +208,17 @@ class SessionClient(adapter.LegacyJsonAdapter):
     def delete(self, url, **kwargs):
         return self._cs_request(url, 'DELETE', **kwargs)
 
+    def _get_base_url(self):
+        endpoint = self.get_endpoint()
+        base_url = '/'.join(endpoint.split('/')[:3]) + '/'
+        return base_url
+
     def get_volume_api_version_from_endpoint(self):
         try:
             version = get_volume_api_from_url(self.get_endpoint())
         except exceptions.UnsupportedVersion as e:
             msg = (_("Service catalog returned invalid url.\n"
-                     "%s") % six.text_type(e.message))
+                     "%s") % six.text_type(e))
             raise exceptions.UnsupportedVersion(msg)
 
         return version
@@ -144,6 +238,16 @@ class SessionClient(adapter.LegacyJsonAdapter):
         raise AttributeError('There is no service catalog for this type of '
                              'auth plugin.')
 
+    def _cs_request_base_url(self, url, method, **kwargs):
+        base_url = self._get_base_url()
+        return self._cs_request(
+            base_url + url,
+            method,
+            **kwargs)
+
+    def get_with_base_url(self, url, **kwargs):
+        return self._cs_request_base_url(url, 'GET', **kwargs)
+
 
 class HTTPClient(object):
 
@@ -157,11 +261,15 @@ class HTTPClient(object):
                  service_name=None, volume_service_name=None,
                  bypass_url=None, retries=None,
                  http_log_debug=False, cacert=None,
-                 auth_system='keystone', auth_plugin=None):
+                 auth_system='keystone', auth_plugin=None, api_version=None,
+                 logger=None, user_domain_name='Default',
+                 project_domain_name='Default', global_request_id=None):
         self.user = user
         self.password = password
         self.projectid = projectid
         self.tenant_id = tenant_id
+        self.api_version = api_version or api_versions.APIVersion()
+        self.global_request_id = global_request_id
 
         if auth_system and auth_system != 'keystone' and not auth_plugin:
             raise exceptions.AuthSystemNotFound(auth_system)
@@ -187,6 +295,8 @@ class HTTPClient(object):
         self.proxy_token = proxy_token
         self.proxy_tenant_id = proxy_tenant_id
         self.timeout = timeout
+        self.user_domain_name = user_domain_name
+        self.project_domain_name = project_domain_name
 
         if insecure:
             self.verify_cert = False
@@ -199,7 +309,7 @@ class HTTPClient(object):
         self.auth_system = auth_system
         self.auth_plugin = auth_plugin
 
-        self._logger = logging.getLogger(__name__)
+        self._logger = logger or logging.getLogger(__name__)
 
     def _safe_header(self, name, value):
         if name in HTTPClient.SENSITIVE_HEADERS:
@@ -228,10 +338,7 @@ class HTTPClient(object):
             string_parts.append(header)
 
         if 'data' in kwargs:
-            if "password" in kwargs['data']:
-                data = strutils.mask_password(kwargs['data'])
-            else:
-                data = kwargs['data']
+            data = strutils.mask_password(kwargs['data'])
             string_parts.append(" -d '%s'" % (data))
         self._logger.debug("\nREQ: %s\n" % "".join(string_parts))
 
@@ -242,7 +349,11 @@ class HTTPClient(object):
             "RESP: [%s] %s\nRESP BODY: %s\n",
             resp.status_code,
             resp.headers,
-            resp.text)
+            strutils.mask_password(resp.text))
+
+        # if service name is None then use service_type for logging
+        service = self.service_name or self.service_type
+        _log_request_id(self._logger, resp, service)
 
     def request(self, url, method, **kwargs):
         kwargs.setdefault('headers', kwargs.get('headers', {}))
@@ -254,8 +365,11 @@ class HTTPClient(object):
 
         if 'body' in kwargs:
             kwargs['headers']['Content-Type'] = 'application/json'
-            kwargs['data'] = json.dumps(kwargs['body'])
-            del kwargs['body']
+            kwargs['data'] = json.dumps(kwargs.pop('body'))
+        api_versions.update_headers(kwargs["headers"], self.api_version)
+
+        if self.global_request_id:
+            kwargs['headers'].setdefault(REQ_ID_HEADER, self.global_request_id)
 
         if self.timeout:
             kwargs.setdefault('timeout', self.timeout)
@@ -291,8 +405,9 @@ class HTTPClient(object):
             if self.projectid:
                 kwargs['headers']['X-Auth-Project-Id'] = self.projectid
             try:
-                resp, body = self.request(self.management_url + url, method,
-                                          **kwargs)
+                if not url.startswith(self.management_url):
+                    url = self.management_url + url
+                resp, body = self.request(url, method, **kwargs)
                 return resp, body
             except exceptions.BadRequest as e:
                 if attempts > self.retries:
@@ -306,6 +421,13 @@ class HTTPClient(object):
                 attempts -= 1
                 auth_attempts += 1
                 continue
+            except exceptions.OverLimit as overlim:
+                if attempts > self.retries or overlim.retry_after < 1:
+                    raise
+                msg = "Retrying after %s seconds." % overlim.retry_after
+                self._logger.debug(msg)
+                sleep(overlim.retry_after)
+                continue
             except exceptions.ClientException as e:
                 if attempts > self.retries:
                     raise
@@ -345,12 +467,12 @@ class HTTPClient(object):
             version = get_volume_api_from_url(self.management_url)
         except exceptions.UnsupportedVersion as e:
             if self.management_url == self.bypass_url:
-                msg = (_("Invalid url was specified in --bypass-url or "
+                msg = (_("Invalid url was specified in --os-endpoint or "
                          "environment variable CINDERCLIENT_BYPASS_URL.\n"
-                         "%s") % six.text_type(e.message))
+                         "%s") % six.text_type(e))
             else:
                 msg = (_("Service catalog returned invalid url.\n"
-                         "%s") % six.text_type(e.message))
+                         "%s") % six.text_type(e))
 
             raise exceptions.UnsupportedVersion(msg)
 
@@ -361,11 +483,11 @@ class HTTPClient(object):
         We may get redirected to another site, fail or actually get
         back a service catalog with a token and our endpoints.
         """
-
-        if resp.status_code == 200:  # content must always present
+        # content must always present
+        if resp.status_code == 200 or resp.status_code == 201:
             try:
                 self.auth_url = url
-                self.auth_ref = access.AccessInfo.factory(resp, body)
+                self.auth_ref = access.create(resp=resp, body=body)
                 self.service_catalog = self.auth_ref.service_catalog
 
                 if extract_token:
@@ -373,7 +495,7 @@ class HTTPClient(object):
 
                 management_url = self.service_catalog.url_for(
                     region_name=self.region_name,
-                    endpoint_type=self.endpoint_type,
+                    interface=self.endpoint_type,
                     service_type=self.service_type,
                     service_name=self.service_name)
                 self.management_url = management_url.rstrip('/')
@@ -382,7 +504,10 @@ class HTTPClient(object):
                 print("Found more than one valid endpoint. Use a more "
                       "restrictive filter")
                 raise
-            except KeyError:
+            except ValueError:
+                # ValueError is raised when you pass an invalid response to
+                # access.create. This should never happen in reality if the
+                # status code is 200.
                 raise exceptions.AuthorizationFailure()
             except exceptions.EndpointNotFound:
                 print("Could not find any suitable endpoint. Correct region?")
@@ -436,12 +561,10 @@ class HTTPClient(object):
                                          path, query, frag))
 
         auth_url = self.auth_url
-        if self.version == "v2.0":
+        if self.version == "v2.0" or self.version == "v3":
             while auth_url:
                 if not self.auth_system or self.auth_system == 'keystone':
-                    auth_url = self._v2_auth(auth_url)
-                else:
-                    auth_url = self._plugin_auth(auth_url)
+                    auth_url = self._v2_or_v3_auth(auth_url)
 
             # Are we acting on behalf of another user via an
             # existing token? If so, our actual endpoints may
@@ -465,7 +588,7 @@ class HTTPClient(object):
             except exceptions.AuthorizationFailure:
                 if auth_url.find('v2.0') < 0:
                     auth_url = auth_url + '/v2.0'
-                self._v2_auth(auth_url)
+                self._v2_or_v3_auth(auth_url)
 
         if self.bypass_url:
             self.set_management_url(self.bypass_url)
@@ -495,26 +618,43 @@ class HTTPClient(object):
         else:
             raise exceptions.from_response(resp, body)
 
-    def _plugin_auth(self, auth_url):
-        return self.auth_plugin.authenticate(self, auth_url)
-
-    def _v2_auth(self, url):
+    def _v2_or_v3_auth(self, url):
         """Authenticate against a v2.0 auth service."""
-        body = {"auth": {
-            "passwordCredentials": {"username": self.user,
-                                    "password": self.password}}}
+        if self.version == "v3":
+            body = {
+                "auth": {
+                    "identity": {
+                        "methods": ["password"],
+                        "password": {"user": {
+                            "domain": {"name": self.user_domain_name},
+                            "name": self.user,
+                            "password": self.password}}},
+                }
+            }
+            scope = {"project": {"domain": {"name": self.project_domain_name}}}
+            if self.projectid:
+                scope['project']['name'] = self.projectid
+            elif self.tenant_id:
+                scope['project']['id'] = self.tenant_id
 
-        if self.projectid:
-            body['auth']['tenantName'] = self.projectid
-        elif self.tenant_id:
-            body['auth']['tenantId'] = self.tenant_id
+            body["auth"]["scope"] = scope
+        else:
+            body = {"auth": {
+                "passwordCredentials": {"username": self.user,
+                                        "password": self.password}}}
 
+            if self.projectid:
+                body['auth']['tenantName'] = self.projectid
+            elif self.tenant_id:
+                body['auth']['tenantId'] = self.tenant_id
         self._authenticate(url, body)
 
     def _authenticate(self, url, body):
         """Authenticate and extract the service catalog."""
-        token_url = url + "/tokens"
-
+        if self.version == 'v3':
+            token_url = url + "/auth/tokens"
+        else:
+            token_url = url + "/tokens"
         # Make sure we follow redirects when trying to reach Keystone
         resp, body = self.request(
             token_url,
@@ -536,11 +676,10 @@ def _construct_http_client(username=None, password=None, project_id=None,
                            auth_system='keystone', auth_plugin=None,
                            cacert=None, tenant_id=None,
                            session=None,
-                           auth=None,
+                           auth=None, api_version=None,
                            **kwargs):
 
-    # Don't use sessions if third party plugin is used
-    if session and not auth_plugin:
+    if session:
         kwargs.setdefault('user_agent', 'python-cinderclient')
         kwargs.setdefault('interface', endpoint_type)
         return SessionClient(session=session,
@@ -548,10 +687,13 @@ def _construct_http_client(username=None, password=None, project_id=None,
                              service_type=service_type,
                              service_name=service_name,
                              region_name=region_name,
+                             retries=retries,
+                             api_version=api_version,
                              **kwargs)
     else:
         # FIXME(jamielennox): username and password are now optional. Need
         # to test that they were provided in this mode.
+        logger = kwargs.get('logger')
         return HTTPClient(username,
                           password,
                           projectid=project_id,
@@ -572,13 +714,28 @@ def _construct_http_client(username=None, password=None, project_id=None,
                           cacert=cacert,
                           auth_system=auth_system,
                           auth_plugin=auth_plugin,
+                          logger=logger,
+                          api_version=api_version
                           )
 
 
+def _get_client_class_and_version(version):
+    if not isinstance(version, api_versions.APIVersion):
+        version = api_versions.get_api_version(version)
+    else:
+        api_versions.check_major_version(version)
+    if version.is_latest():
+        raise exceptions.UnsupportedVersion(
+            _("The version should be explicit, not latest."))
+    return version, importutils.import_class(
+        "cinderclient.v%s.client.Client" % version.ver_major)
+
+
 def get_client_class(version):
     version_map = {
         '1': 'cinderclient.v1.client.Client',
         '2': 'cinderclient.v2.client.Client',
+        '3': 'cinderclient.v3.client.Client',
     }
     try:
         client_path = version_map[str(version)]
@@ -604,7 +761,7 @@ def discover_extensions(version):
 
 def _discover_via_python_path():
     for (module_loader, name, ispkg) in pkgutil.iter_modules():
-        if name.endswith('python_cinderclient_ext'):
+        if name.endswith('cinderclient_ext'):
             if not hasattr(module_loader, 'load_module'):
                 # Python 2.6 compat: actually get an ImpImporter obj
                 module_loader = module_loader.find_module(name)
@@ -630,5 +787,28 @@ def _discover_via_contrib_path(version):
 
 
 def Client(version, *args, **kwargs):
-    client_class = get_client_class(version)
-    return client_class(*args, **kwargs)
+    """Initialize client object based on given version.
+
+    HOW-TO:
+    The simplest way to create a client instance is initialization with your
+    credentials::
+
+    .. code-block:: python
+
+        >>> from cinderclient import client
+        >>> cinder = client.Client(VERSION, USERNAME, PASSWORD,
+        ...                      PROJECT_NAME, AUTH_URL)
+
+    Here ``VERSION`` can be a string or
+    ``cinderclient.api_versions.APIVersion`` obj. If you prefer string value,
+    you can use ``1`` (deprecated now), ``2``, or ``3.X``
+    (where X is a microversion).
+
+
+    Alternatively, you can create a client instance using the keystoneclient
+    session API. See "The cinderclient Python API" page at
+    python-cinderclient's doc.
+    """
+    api_version, client_class = _get_client_class_and_version(version)
+    return client_class(api_version=api_version,
+                        *args, **kwargs)
