@@ -14,10 +14,7 @@
 
 from datetime import datetime
 
-try:
-    import urlparse
-except ImportError:
-    import urllib.parse as urlparse
+import six.moves.urllib.parse as urlparse
 
 from cinderclient import client as base_client
 from cinderclient.tests.unit import fakes
@@ -31,7 +28,9 @@ REQUEST_ID = 'req-test-request-id'
 def _stub_volume(*args, **kwargs):
     volume = {
         "migration_status": None,
-        "attachments": [{'server_id': 1234}],
+        "attachments": [{u'server_id': u'1234',
+                         u'id': u'3f88836f-adde-4296-9f6b-2c59a0bcda9a',
+                         u'attachment_id': u'5678'}],
         "links": [
             {
                 "href": "http://localhost/v2/fake/volumes/1234",
@@ -252,12 +251,72 @@ def _stub_extend(id, new_size):
     return {'volume_id': '712f4980-5ac1-41e5-9383-390aa7c9f58b'}
 
 
+def _stub_server_versions():
+    return [
+        {
+            "status": "SUPPORTED",
+            "updated": "2015-07-30T11:33:21Z",
+            "links": [
+                {
+                    "href": "http://docs.openstack.org/",
+                    "type": "text/html",
+                    "rel": "describedby",
+                },
+                {
+                    "href": "http://localhost:8776/v1/",
+                    "rel": "self",
+                }
+            ],
+            "min_version": "",
+            "version": "",
+            "id": "v1.0",
+        },
+        {
+            "status": "SUPPORTED",
+            "updated": "2015-09-30T11:33:21Z",
+            "links": [
+                {
+                    "href": "http://docs.openstack.org/",
+                    "type": "text/html",
+                    "rel": "describedby",
+                },
+                {
+                    "href": "http://localhost:8776/v2/",
+                    "rel": "self",
+                }
+            ],
+            "min_version": "",
+            "version": "",
+            "id": "v2.0",
+        },
+        {
+            "status": "CURRENT",
+            "updated": "2016-04-01T11:33:21Z",
+            "links": [
+                {
+                    "href": "http://docs.openstack.org/",
+                    "type": "text/html",
+                    "rel": "describedby",
+                },
+                {
+                    "href": "http://localhost:8776/v3/",
+                    "rel": "self",
+                }
+            ],
+            "min_version": "3.0",
+            "version": "3.1",
+            "id": "v3.0",
+        }
+    ]
+
+
 class FakeClient(fakes.FakeClient, client.Client):
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, api_version=None, *args, **kwargs):
         client.Client.__init__(self, 'username', 'password',
                                'project_id', 'auth_url',
                                extensions=kwargs.get('extensions'))
+        self.api_version = api_version
         self.client = FakeHTTPClient(**kwargs)
 
     def get_volume_api_version_from_endpoint(self):
@@ -266,7 +325,7 @@ class FakeClient(fakes.FakeClient, client.Client):
 
 class FakeHTTPClient(base_client.HTTPClient):
 
-    def __init__(self, **kwargs):
+    def __init__(self, version_header=None, **kwargs):
         self.username = 'username'
         self.password = 'password'
         self.auth_url = 'auth_url'
@@ -274,6 +333,7 @@ class FakeHTTPClient(base_client.HTTPClient):
         self.management_url = 'http://10.0.2.15:8776/v2/fake'
         self.osapi_max_limit = 1000
         self.marker = None
+        self.version_header = version_header
 
     def _cs_request(self, url, method, **kwargs):
         # Check that certain things are called correctly
@@ -310,6 +370,8 @@ class FakeHTTPClient(base_client.HTTPClient):
         status, headers, body = getattr(self, callback)(**kwargs)
         # add fake request-id header
         headers['x-openstack-request-id'] = REQUEST_ID
+        if self.version_header:
+            headers['OpenStack-API-version'] = self.version_header
         r = utils.TestResponse({
             "status_code": status,
             "text": body,
@@ -327,9 +389,12 @@ class FakeHTTPClient(base_client.HTTPClient):
     #
 
     def get_snapshots_detail(self, **kw):
+        if kw.get('with_count', False):
+            return (200, {}, {'snapshots': [
+                _stub_snapshot(),
+            ], 'count': 1})
         return (200, {}, {'snapshots': [
-            _stub_snapshot(),
-        ]})
+            _stub_snapshot()]})
 
     def get_snapshots_1234(self, **kw):
         return (200, {}, {'snapshot': _stub_snapshot(id='1234')})
@@ -358,6 +423,8 @@ class FakeHTTPClient(base_client.HTTPClient):
             assert 'status' in body['os-reset_status']
         elif action == 'os-update_snapshot_status':
             assert 'status' in body['os-update_snapshot_status']
+        elif action == 'os-force_delete':
+            assert body[action] is None
         elif action == 'os-unmanage':
             assert body[action] is None
         else:
@@ -400,6 +467,10 @@ class FakeHTTPClient(base_client.HTTPClient):
             ]})
 
     def get_volumes_detail(self, **kw):
+        if kw.get('with_count', False):
+            return (200, {}, {"volumes": [
+                _stub_volume(id=kw.get('id', 1234))
+            ], "count": 1})
         return (200, {}, {"volumes": [
             _stub_volume(id=kw.get('id', 1234))
         ]})
@@ -412,6 +483,10 @@ class FakeHTTPClient(base_client.HTTPClient):
         r = {'volume': self.get_volumes_detail(id=5678)[2]['volumes'][0]}
         return (200, {}, r)
 
+    def get_volumes_1234_metadata(self, **kw):
+        r = {"metadata": {'k1': 'v1', 'k2': 'v2', 'k3': 'v3'}}
+        return (200, {}, r)
+
     def get_volumes_1234_encryption(self, **kw):
         r = {'encryption_key_id': 'id'}
         return (200, {}, r)
@@ -441,7 +516,8 @@ class FakeHTTPClient(base_client.HTTPClient):
         elif action == 'os-roll_detaching':
             assert body[action] is None
         elif action == 'os-reset_status':
-            assert 'status' in body[action]
+            assert ('status' or 'attach_status' or 'migration_status'
+                    in body[action])
         elif action == 'os-extend':
             assert list(body[action]) == ['new_size']
         elif action == 'os-migrate_volume':
@@ -465,10 +541,24 @@ class FakeHTTPClient(base_client.HTTPClient):
             assert 'key' in body[action]
         elif action == 'os-show_image_metadata':
             assert body[action] is None
+        elif action == 'os-volume_upload_image':
+            assert 'image_name' in body[action]
+            _body = body
+        elif action == 'revert':
+            assert 'snapshot_id' in body[action]
         else:
             raise AssertionError("Unexpected action: %s" % action)
         return (resp, {}, _body)
 
+    def get_volumes_fake(self, **kw):
+        r = {'volume': self.get_volumes_detail(id='fake')[2]['volumes'][0]}
+        return (200, {}, r)
+
+    def post_volumes_fake_action(self, body, **kw):
+        _body = None
+        resp = 202
+        return (resp, {}, _body)
+
     def post_volumes_5678_action(self, body, **kw):
         return self.post_volumes_1234_action(body, **kw)
 
@@ -555,7 +645,6 @@ class FakeHTTPClient(base_client.HTTPClient):
                           'gigabytes': 1,
                           'backups': 1,
                           'backup_gigabytes': 1,
-                          'consistencygroups': 1,
                           'per_volume_gigabytes': 1, }})
 
     def get_os_quota_sets_test_defaults(self):
@@ -567,7 +656,6 @@ class FakeHTTPClient(base_client.HTTPClient):
                           'gigabytes': 1,
                           'backups': 1,
                           'backup_gigabytes': 1,
-                          'consistencygroups': 1,
                           'per_volume_gigabytes': 1, }})
 
     def put_os_quota_sets_test(self, body, **kw):
@@ -582,7 +670,6 @@ class FakeHTTPClient(base_client.HTTPClient):
                           'gigabytes': 1,
                           'backups': 1,
                           'backup_gigabytes': 1,
-                          'consistencygroups': 2,
                           'per_volume_gigabytes': 1, }})
 
     def delete_os_quota_sets_1234(self, **kw):
@@ -598,28 +685,22 @@ class FakeHTTPClient(base_client.HTTPClient):
     def get_os_quota_class_sets_test(self, **kw):
         return (200, {}, {'quota_class_set': {
                           'class_name': 'test',
-                          'metadata_items': [],
                           'volumes': 1,
                           'snapshots': 1,
                           'gigabytes': 1,
                           'backups': 1,
                           'backup_gigabytes': 1,
-                          'consistencygroups': 1,
                           'per_volume_gigabytes': 1, }})
 
     def put_os_quota_class_sets_test(self, body, **kw):
         assert list(body) == ['quota_class_set']
-        fakes.assert_has_keys(body['quota_class_set'],
-                              required=['class_name'])
+        fakes.assert_has_keys(body['quota_class_set'])
         return (200, {}, {'quota_class_set': {
-                          'class_name': 'test',
-                          'metadata_items': [],
                           'volumes': 2,
                           'snapshots': 2,
                           'gigabytes': 1,
                           'backups': 1,
                           'backup_gigabytes': 1,
-                          'consistencygroups': 2,
                           'per_volume_gigabytes': 1}})
 
     #
@@ -641,7 +722,7 @@ class FakeHTTPClient(base_client.HTTPClient):
         return (200, {}, {'volume_type': {'id': 1,
                           'name': 'test-type-1',
                           'description': 'test_type-1-desc',
-                          'extra_specs': {}}})
+                          'extra_specs': {u'key': u'value'}}})
 
     def get_types_2(self, **kw):
         return (200, {}, {'volume_type': {'id': 2,
@@ -685,12 +766,28 @@ class FakeHTTPClient(base_client.HTTPClient):
     def delete_types_1_extra_specs_k(self, **kw):
         return(204, {}, None)
 
+    def delete_types_1_extra_specs_m(self, **kw):
+        return(204, {}, None)
+
     def delete_types_1(self, **kw):
         return (202, {}, None)
 
+    def delete_types_3_extra_specs_k(self, **kw):
+        return(204, {}, None)
+
+    def delete_types_3(self, **kw):
+        return (202, {}, None)
+
     def put_types_1(self, **kw):
         return self.get_types_1()
 
+    def put_types_3(self, **kw):
+        return (200, {}, {'volume_type': {'id': 3,
+                          'name': 'test-type-2',
+                          'description': 'test_type-3-desc',
+                          'is_public': True,
+                          'extra_specs': {}}})
+
     #
     # VolumeAccess
     #
@@ -795,6 +892,12 @@ class FakeHTTPClient(base_client.HTTPClient):
         tenant_id = '0fa851f6668144cf9cd8c8419c1646c1'
         backup1 = '76a17945-3c6f-435c-975b-b5685db10b62'
         backup2 = 'd09534c6-08b8-4441-9e87-8976f3a8f699'
+        if kw.get('with_count', False):
+            return (200, {},
+                    {'backups': [
+                        _stub_backup_full(backup1, base_uri, tenant_id),
+                        _stub_backup_full(backup2, base_uri, tenant_id)],
+                    'count': 2})
         return (200, {},
                 {'backups': [
                     _stub_backup_full(backup1, base_uri, tenant_id),
@@ -956,6 +1059,10 @@ class FakeHTTPClient(base_client.HTTPClient):
         return (200, {},
                 {'transfer': _stub_transfer(transfer1, base_uri, tenant_id)})
 
+    def get_with_base_url(self, url, **kw):
+        server_versions = _stub_server_versions()
+        return (200, {'versions': server_versions})
+
     #
     # Services
     #
@@ -989,9 +1096,9 @@ class FakeHTTPClient(base_client.HTTPClient):
             },
         ]
         if host:
-            services = filter(lambda i: i['host'] == host, services)
+            services = [i for i in services if i['host'] == host]
         if binary:
-            services = filter(lambda i: i['binary'] == binary, services)
+            services = [i for i in services if i['binary'] == binary]
         return (200, {}, {'services': services})
 
     def put_os_services_enable(self, body, **kw):
@@ -1077,11 +1184,58 @@ class FakeHTTPClient(base_client.HTTPClient):
     def put_snapshots_1234_metadata(self, **kw):
         return (200, {}, {"metadata": {"key1": "val1", "key2": "val2"}})
 
+    def get_os_volume_manage(self, **kw):
+        vol_id = "volume-ffffffff-0000-ffff-0000-ffffffffffff"
+        vols = [{"size": 4, "safe_to_manage": False, "actual_size": 4.0,
+                 "reference": {"source-name": vol_id}},
+                {"size": 5, "safe_to_manage": True, "actual_size": 4.3,
+                 "reference": {"source-name": "myvol"}}]
+        return (200, {}, {"manageable-volumes": vols})
+
+    def get_os_volume_manage_detail(self, **kw):
+        vol_id = "volume-ffffffff-0000-ffff-0000-ffffffffffff"
+        vols = [{"size": 4, "reason_not_safe": "volume in use",
+                 "safe_to_manage": False, "extra_info": "qos_setting:high",
+                 "reference": {"source-name": vol_id},
+                 "actual_size": 4.0},
+                {"size": 5, "reason_not_safe": None, "safe_to_manage": True,
+                 "extra_info": "qos_setting:low", "actual_size": 4.3,
+                 "reference": {"source-name": "myvol"}}]
+        return (200, {}, {"manageable-volumes": vols})
+
     def post_os_volume_manage(self, **kw):
         volume = _stub_volume(id='1234')
         volume.update(kw['body']['volume'])
         return (202, {}, {'volume': volume})
 
+    def get_os_snapshot_manage(self, **kw):
+        snap_id = "snapshot-ffffffff-0000-ffff-0000-ffffffffffff"
+        snaps = [{"actual_size": 4.0, "size": 4,
+                 "safe_to_manage": False, "source_id_type": "source-name",
+                 "source_cinder_id": "00000000-ffff-0000-ffff-00000000",
+                 "reference": {"source-name": snap_id},
+                 "source_identifier": "volume-00000000-ffff-0000-ffff-000000"},
+                {"actual_size": 4.3, "reference": {"source-name": "mysnap"},
+                 "source_id_type": "source-name", "source_identifier": "myvol",
+                 "safe_to_manage": True, "source_cinder_id": None, "size": 5}]
+        return (200, {}, {"manageable-snapshots": snaps})
+
+    def get_os_snapshot_manage_detail(self, **kw):
+        snap_id = "snapshot-ffffffff-0000-ffff-0000-ffffffffffff"
+        snaps = [{"actual_size": 4.0, "size": 4,
+                 "safe_to_manage": False, "source_id_type": "source-name",
+                 "source_cinder_id": "00000000-ffff-0000-ffff-00000000",
+                 "reference": {"source-name": snap_id},
+                 "source_identifier": "volume-00000000-ffff-0000-ffff-000000",
+                 "extra_info": "qos_setting:high",
+                 "reason_not_safe": "snapshot in use"},
+                {"actual_size": 4.3, "reference": {"source-name": "mysnap"},
+                 "safe_to_manage": True, "source_cinder_id": None,
+                 "source_id_type": "source-name", "identifier": "mysnap",
+                 "source_identifier": "myvol", "size": 5,
+                 "extra_info": "qos_setting:low", "reason_not_safe": None}]
+        return (200, {}, {"manageable-snapshots": snaps})
+
     def post_os_snapshot_manage(self, **kw):
         snapshot = _stub_snapshot(id='1234', volume_id='volume_id1')
         snapshot.update(kw['body']['snapshot'])
@@ -1124,9 +1278,9 @@ class FakeHTTPClient(base_client.HTTPClient):
                 'storage_protocol': 'iSCSI',
                 'properties': {
                     'compression': {
-                        'title': 'Compression',
-                        'description': 'Enables compression.',
-                        'type': 'boolean'},
+                        u'title': u'Compression',
+                        u'description': u'Enables compression.',
+                        u'type': u'boolean'},
                 }
             }
         )
