@@ -13,6 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import ddt
 import fixtures
 import mock
 from requests_mock.contrib import fixture as requests_mock_fixture
@@ -21,13 +22,17 @@ from six.moves.urllib import parse
 from cinderclient import client
 from cinderclient import exceptions
 from cinderclient import shell
-from cinderclient.v2 import volumes
 from cinderclient.v2 import shell as test_shell
+from cinderclient.v2 import volume_backups
+from cinderclient.v2 import volumes
+
+from cinderclient.tests.unit.fixture_data import keystone_client
 from cinderclient.tests.unit import utils
 from cinderclient.tests.unit.v2 import fakes
-from cinderclient.tests.unit.fixture_data import keystone_client
 
 
+@ddt.ddt
+@mock.patch.object(client, 'Client', fakes.FakeClient)
 class ShellTest(utils.TestCase):
 
     FAKE_ENV = {
@@ -48,10 +53,6 @@ class ShellTest(utils.TestCase):
 
         self.shell = shell.OpenStackCinderShell()
 
-        # HACK(bcwaldon): replace this when we start using stubs
-        self.old_get_client_class = client.get_client_class
-        client.get_client_class = lambda *_: fakes.FakeClient
-
         self.requests = self.useFixture(requests_mock_fixture.Fixture())
         self.requests.register_uri(
             'GET', keystone_client.BASE_URL,
@@ -66,18 +67,6 @@ class ShellTest(utils.TestCase):
 
         return Args(args)
 
-    def tearDown(self):
-        # For some methods like test_image_meta_bad_action we are
-        # testing a SystemExit to be thrown and object self.shell has
-        # no time to get instantiated, which is OK in this case, so
-        # we make sure the method is there before launching it.
-        if hasattr(self.shell, 'cs'):
-            self.shell.cs.clear_callstack()
-
-        # HACK(bcwaldon): replace this when we start using stubs
-        client.get_client_class = self.old_get_client_class
-        super(ShellTest, self).tearDown()
-
     def run_command(self, cmd):
         self.shell.main(cmd.split())
 
@@ -86,11 +75,6 @@ class ShellTest(utils.TestCase):
         return self.shell.cs.assert_called(method, url, body,
                                            partial_body, **kwargs)
 
-    def assert_called_anytime(self, method, url, body=None,
-                              partial_body=None):
-        return self.shell.cs.assert_called_anytime(method, url, body,
-                                                   partial_body)
-
     def test_list(self):
         self.run_command('list')
         # NOTE(jdg): we default to detail currently
@@ -110,13 +94,8 @@ class ShellTest(utils.TestCase):
         self.run_command('create --metadata key1="--test1" 1')
         self.assert_called('GET', '/volumes/1234')
         expected = {'volume': {'imageRef': None,
-                               'project_id': None,
-                               'status': 'creating',
                                'size': 1,
-                               'user_id': None,
                                'availability_zone': None,
-                               'source_replica': None,
-                               'attach_status': 'detached',
                                'source_volid': None,
                                'consistencygroup_id': None,
                                'name': None,
@@ -131,13 +110,8 @@ class ShellTest(utils.TestCase):
         self.run_command('create --metadata key1="--t1" --name="t" 1')
         self.assert_called('GET', '/volumes/1234')
         expected = {'volume': {'imageRef': None,
-                               'project_id': None,
-                               'status': 'creating',
                                'size': 1,
-                               'user_id': None,
                                'availability_zone': None,
-                               'source_replica': None,
-                               'attach_status': 'detached',
                                'source_volid': None,
                                'consistencygroup_id': None,
                                'name': '"t"',
@@ -151,13 +125,8 @@ class ShellTest(utils.TestCase):
     def test_delimit_metadata_args(self):
         self.run_command('create --metadata key1="test1" key2="test2" 1')
         expected = {'volume': {'imageRef': None,
-                               'project_id': None,
-                               'status': 'creating',
                                'size': 1,
-                               'user_id': None,
                                'availability_zone': None,
-                               'source_replica': None,
-                               'attach_status': 'detached',
                                'source_volid': None,
                                'consistencygroup_id': None,
                                'name': None,
@@ -173,13 +142,8 @@ class ShellTest(utils.TestCase):
         self.run_command('create --metadata key1="t1" --name="t" 1')
         self.assert_called('GET', '/volumes/1234')
         expected = {'volume': {'imageRef': None,
-                               'project_id': None,
-                               'status': 'creating',
                                'size': 1,
-                               'user_id': None,
                                'availability_zone': None,
-                               'source_replica': None,
-                               'attach_status': 'detached',
                                'source_volid': None,
                                'consistencygroup_id': None,
                                'name': '"t"',
@@ -194,6 +158,14 @@ class ShellTest(utils.TestCase):
         self.run_command('list --status=available')
         self.assert_called('GET', '/volumes/detail?status=available')
 
+    def test_list_filter_bootable_true(self):
+        self.run_command('list --bootable=true')
+        self.assert_called('GET', '/volumes/detail?bootable=true')
+
+    def test_list_filter_bootable_false(self):
+        self.run_command('list --bootable=false')
+        self.assert_called('GET', '/volumes/detail?bootable=false')
+
     def test_list_filter_name(self):
         self.run_command('list --name=1234')
         self.assert_called('GET', '/volumes/detail?name=1234')
@@ -210,6 +182,41 @@ class ShellTest(utils.TestCase):
         self.run_command('list --limit=10')
         self.assert_called('GET', '/volumes/detail?limit=10')
 
+    @mock.patch("cinderclient.utils.print_list")
+    def test_list_field(self, mock_print):
+        self.run_command('list --field Status,Name,Size,Bootable')
+        self.assert_called('GET', '/volumes/detail')
+        key_list = ['ID', 'Status', 'Name', 'Size', 'Bootable']
+        mock_print.assert_called_once_with(mock.ANY, key_list,
+            exclude_unavailable=True, sortby_index=0)
+
+    @mock.patch("cinderclient.utils.print_list")
+    def test_list_field_with_all_tenants(self, mock_print):
+        self.run_command('list --field Status,Name,Size,Bootable '
+                         '--all-tenants 1')
+        self.assert_called('GET', '/volumes/detail?all_tenants=1')
+        key_list = ['ID', 'Status', 'Name', 'Size', 'Bootable']
+        mock_print.assert_called_once_with(mock.ANY, key_list,
+            exclude_unavailable=True, sortby_index=0)
+
+    @mock.patch("cinderclient.utils.print_list")
+    def test_list_duplicate_fields(self, mock_print):
+        self.run_command('list --field Status,id,Size,status')
+        self.assert_called('GET', '/volumes/detail')
+        key_list = ['ID', 'Status', 'Size']
+        mock_print.assert_called_once_with(mock.ANY, key_list,
+            exclude_unavailable=True, sortby_index=0)
+
+    @mock.patch("cinderclient.utils.print_list")
+    def test_list_field_with_tenant(self, mock_print):
+        self.run_command('list --field Status,Name,Size,Bootable '
+                         '--tenant 123')
+        self.assert_called('GET',
+            '/volumes/detail?all_tenants=1&project_id=123')
+        key_list = ['ID', 'Status', 'Name', 'Size', 'Bootable']
+        mock_print.assert_called_once_with(mock.ANY, key_list,
+            exclude_unavailable=True, sortby_index=0)
+
     def test_list_sort_valid(self):
         self.run_command('list --sort_key=id --sort_dir=asc')
         self.assert_called('GET', '/volumes/detail?sort_dir=asc&sort_key=id')
@@ -318,29 +325,38 @@ class ShellTest(utils.TestCase):
         self.assert_called_anytime('POST', '/volumes', partial_body=expected)
         self.assert_called('GET', '/volumes/1234')
 
-    def test_create_volume_from_replica(self):
-        expected = {'volume': {'size': None}}
-
-        expected['volume']['source_replica'] = '1234'
-        self.run_command('create --source-replica=1234')
-        self.assert_called_anytime('POST', '/volumes', partial_body=expected)
-        self.assert_called('GET', '/volumes/1234')
-
     def test_create_volume_from_image(self):
-        expected = {'volume': {'status': 'creating',
-                               'size': 1,
-                               'imageRef': '1234',
-                               'attach_status': 'detached'}}
+        expected = {'volume': {'size': 1,
+                               'imageRef': '1234'}}
         self.run_command('create --image=1234 1')
         self.assert_called_anytime('POST', '/volumes', partial_body=expected)
         self.assert_called('GET', '/volumes/1234')
 
+    def test_upload_to_image(self):
+        expected = {'os-volume_upload_image': {'force': False,
+                                               'container_format': 'bare',
+                                               'disk_format': 'raw',
+                                               'image_name': 'test-image'}}
+        self.run_command('upload-to-image 1234 test-image')
+        self.assert_called_anytime('GET', '/volumes/1234')
+        self.assert_called_anytime('POST', '/volumes/1234/action',
+                                   body=expected)
+
+    def test_upload_to_image_force(self):
+        expected = {'os-volume_upload_image': {'force': 'True',
+                                               'container_format': 'bare',
+                                               'disk_format': 'raw',
+                                               'image_name': 'test-image'}}
+        self.run_command('upload-to-image --force=True 1234 test-image')
+        self.assert_called_anytime('GET', '/volumes/1234')
+        self.assert_called_anytime('POST', '/volumes/1234/action',
+                                   body=expected)
+
     def test_create_size_required_if_not_snapshot_or_clone(self):
         self.assertRaises(SystemExit, self.run_command, 'create')
 
     def test_create_size_zero_if_not_snapshot_or_clone(self):
-        expected = {'volume': {'status': 'creating',
-                               'size': 0}}
+        expected = {'volume': {'size': 0}}
         self.run_command('create 0')
         self.assert_called_anytime('POST', '/volumes', partial_body=expected)
         self.assert_called('GET', '/volumes/1234')
@@ -364,6 +380,16 @@ class ShellTest(utils.TestCase):
         self.assert_called_anytime('DELETE', '/volumes/1234')
         self.assert_called('DELETE', '/volumes/5678')
 
+    def test_delete_with_cascade_true(self):
+        self.run_command('delete 1234 --cascade')
+        self.assert_called('DELETE', '/volumes/1234?cascade=True')
+        self.run_command('delete --cascade 1234')
+        self.assert_called('DELETE', '/volumes/1234?cascade=True')
+
+    def test_delete_with_cascade_with_invalid_value(self):
+        self.assertRaises(SystemExit, self.run_command,
+                          'delete 1234 --cascade 1234')
+
     def test_backup(self):
         self.run_command('backup-create 1234')
         self.assert_called('POST', '/backups')
@@ -376,20 +402,46 @@ class ShellTest(utils.TestCase):
         self.run_command('backup-create 1234 --force')
         self.assert_called('POST', '/backups')
 
+    def test_backup_snapshot(self):
+        self.run_command('backup-create 1234 --snapshot-id 4321')
+        self.assert_called('POST', '/backups')
+
+    def test_multiple_backup_delete(self):
+        self.run_command('backup-delete 1234 5678')
+        self.assert_called_anytime('DELETE', '/backups/1234')
+        self.assert_called('DELETE', '/backups/5678')
+
     def test_restore(self):
         self.run_command('backup-restore 1234')
         self.assert_called('POST', '/backups/1234/restore')
 
+    def test_restore_with_name(self):
+        self.run_command('backup-restore 1234 --name restore_vol')
+        expected = {'restore': {'volume_id': None, 'name': 'restore_vol'}}
+        self.assert_called('POST', '/backups/1234/restore',
+                           body=expected)
+
+    def test_restore_with_name_error(self):
+        self.assertRaises(exceptions.CommandError, self.run_command,
+                          'backup-restore 1234 --volume fake_vol --name '
+                          'restore_vol')
+
+    @ddt.data('backup_name', '1234')
+    @mock.patch('cinderclient.shell_utils.find_backup')
     @mock.patch('cinderclient.utils.print_dict')
     @mock.patch('cinderclient.utils.find_volume')
-    def test_do_backup_restore(self,
-                               mock_find_volume,
-                               mock_print_dict):
+    def test_do_backup_restore_with_name(self,
+                                         value,
+                                         mock_find_volume,
+                                         mock_print_dict,
+                                         mock_find_backup):
         backup_id = '1234'
         volume_id = '5678'
+        name = None
         input = {
-            'backup': backup_id,
-            'volume': volume_id
+            'backup': value,
+            'volume': volume_id,
+            'name': None
         }
 
         args = self._make_args(input)
@@ -397,11 +449,19 @@ class ShellTest(utils.TestCase):
                                'restore') as mocked_restore:
             mock_find_volume.return_value = volumes.Volume(self,
                                                            {'id': volume_id},
-                                                           loaded = True)
+                                                           loaded=True)
+            mock_find_backup.return_value = volume_backups.VolumeBackup(
+                self,
+                {'id': backup_id},
+                loaded=True)
             test_shell.do_backup_restore(self.cs, args)
+            mock_find_backup.assert_called_once_with(
+                self.cs,
+                value)
             mocked_restore.assert_called_once_with(
-                input['backup'],
-                volume_id)
+                backup_id,
+                volume_id,
+                name)
             self.assertTrue(mock_print_dict.called)
 
     def test_record_export(self):
@@ -423,6 +483,28 @@ class ShellTest(utils.TestCase):
         self.assert_called('GET', '/snapshots/detail?'
                            'status=available&volume_id=1234')
 
+    def test_snapshot_list_filter_name(self):
+        self.run_command('snapshot-list --name abc')
+        self.assert_called('GET', '/snapshots/detail?name=abc')
+
+    @mock.patch("cinderclient.utils.print_list")
+    def test_snapshot_list_sort(self, mock_print_list):
+        self.run_command('snapshot-list --sort id')
+        self.assert_called('GET', '/snapshots/detail?sort=id')
+        columns = ['ID', 'Volume ID', 'Status', 'Name', 'Size']
+        mock_print_list.assert_called_once_with(mock.ANY, columns,
+            sortby_index=None)
+
+    def test_snapshot_list_filter_tenant_with_all_tenants(self):
+        self.run_command('snapshot-list --all-tenants=1 --tenant 123')
+        self.assert_called('GET',
+                           '/snapshots/detail?all_tenants=1&project_id=123')
+
+    def test_snapshot_list_filter_tenant_without_all_tenants(self):
+        self.run_command('snapshot-list --tenant 123')
+        self.assert_called('GET',
+                           '/snapshots/detail?all_tenants=1&project_id=123')
+
     def test_rename(self):
         # basic rename with positional arguments
         self.run_command('rename 1234 new-name')
@@ -444,6 +526,14 @@ class ShellTest(utils.TestCase):
         # Call rename with no arguments
         self.assertRaises(SystemExit, self.run_command, 'rename')
 
+    def test_rename_invalid_args(self):
+        """Ensure that error generated does not reference an HTTP code."""
+
+        self.assertRaisesRegexp(exceptions.ClientException,
+                                '(?!HTTP)',
+                                self.run_command,
+                                'rename volume-1234-abcd')
+
     def test_rename_snapshot(self):
         # basic rename with positional arguments
         self.run_command('snapshot-rename 1234 new-name')
@@ -466,6 +556,11 @@ class ShellTest(utils.TestCase):
         # Call snapshot-rename with no arguments
         self.assertRaises(SystemExit, self.run_command, 'snapshot-rename')
 
+    def test_rename_snapshot_invalid_args(self):
+        self.assertRaises(exceptions.ClientException,
+                          self.run_command,
+                          'snapshot-rename snapshot-1234')
+
     def test_set_metadata_set(self):
         self.run_command('metadata 1234 set key1=val1 key2=val2')
         self.assert_called('POST', '/volumes/1234/metadata',
@@ -498,8 +593,7 @@ class ShellTest(utils.TestCase):
 
     def test_reset_state_with_attach_status(self):
         self.run_command('reset-state --attach-status detached 1234')
-        expected = {'os-reset_status': {'status': 'available',
-                                        'attach_status': 'detached'}}
+        expected = {'os-reset_status': {'attach_status': 'detached'}}
         self.assert_called('POST', '/volumes/1234/action', body=expected)
 
     def test_reset_state_with_attach_status_with_flag(self):
@@ -511,8 +605,7 @@ class ShellTest(utils.TestCase):
 
     def test_reset_state_with_reset_migration_status(self):
         self.run_command('reset-state --reset-migration-status 1234')
-        expected = {'os-reset_status': {'status': 'available',
-                                        'migration_status': 'none'}}
+        expected = {'os-reset_status': {'migration_status': 'none'}}
         self.assert_called('POST', '/volumes/1234/action', body=expected)
 
     def test_reset_state_multiple(self):
@@ -552,10 +645,32 @@ class ShellTest(utils.TestCase):
         self.assert_called_anytime('POST', '/snapshots/5678/action',
                                    body=expected)
 
+    def test_backup_reset_state(self):
+        self.run_command('backup-reset-state 1234')
+        expected = {'os-reset_status': {'status': 'available'}}
+        self.assert_called('POST', '/backups/1234/action', body=expected)
+
+    def test_backup_reset_state_with_flag(self):
+        self.run_command('backup-reset-state --state error 1234')
+        expected = {'os-reset_status': {'status': 'error'}}
+        self.assert_called('POST', '/backups/1234/action', body=expected)
+
+    def test_backup_reset_state_multiple(self):
+        self.run_command('backup-reset-state 1234 5678')
+        expected = {'os-reset_status': {'status': 'available'}}
+        self.assert_called_anytime('POST', '/backups/1234/action',
+                                   body=expected)
+        self.assert_called_anytime('POST', '/backups/5678/action',
+                                   body=expected)
+
     def test_type_list(self):
         self.run_command('type-list')
         self.assert_called_anytime('GET', '/types?is_public=None')
 
+    def test_type_show(self):
+        self.run_command('type-show 1')
+        self.assert_called('GET', '/types/1')
+
     def test_type_create(self):
         self.run_command('type-create test-type-1')
         self.assert_called('POST', '/types')
@@ -578,6 +693,33 @@ class ShellTest(utils.TestCase):
                          '--is-public=False')
         self.assert_called('POST', '/types', body=expected)
 
+    def test_type_create_with_invalid_bool(self):
+        self.assertRaises(ValueError,
+                          self.run_command,
+                          ('type-create test-type-3 '
+                          '--description=test_type-3-desc '
+                          '--is-public=invalid_bool'))
+
+    def test_type_update(self):
+        expected = {'volume_type': {'name': 'test-type-1',
+                                    'description': 'test_type-1-desc',
+                                    'is_public': False}}
+        self.run_command('type-update --name test-type-1 '
+                         '--description=test_type-1-desc '
+                         '--is-public=False 1')
+        self.assert_called('PUT', '/types/1', body=expected)
+
+    def test_type_update_with_invalid_bool(self):
+        self.assertRaises(ValueError,
+                          self.run_command,
+                          'type-update --name test-type-1 '
+                          '--description=test_type-1-desc '
+                          '--is-public=invalid_bool 1')
+
+    def test_type_update_without_args(self):
+        self.assertRaises(exceptions.CommandError, self.run_command,
+                          'type-update 1')
+
     def test_type_access_list(self):
         self.run_command('type-access-list --volume-type 3')
         self.assert_called('GET', '/types/3/os-volume-type-access')
@@ -607,6 +749,20 @@ class ShellTest(utils.TestCase):
         self.assert_called('POST', '/types/3/action',
                            body=expected)
 
+    def test_type_delete(self):
+        self.run_command('type-delete 1')
+        self.assert_called('DELETE', '/types/1')
+
+    def test_type_delete_multiple(self):
+        self.run_command('type-delete 1 3')
+        self.assert_called_anytime('DELETE', '/types/1')
+        self.assert_called('DELETE', '/types/3')
+
+    def test_type_delete_by_name(self):
+        self.run_command('type-delete test-type-1')
+        self.assert_called_anytime('GET', '/types?is_public=None')
+        self.assert_called('DELETE', '/types/1')
+
     def test_encryption_type_list(self):
         """
         Test encryption-type-list shell command.
@@ -648,6 +804,17 @@ class ShellTest(utils.TestCase):
         self.assert_called('POST', '/types/2/encryption', body=expected)
         self.assert_called_anytime('GET', '/types/2')
 
+    @ddt.data('--key-size 512 --control-location front-end',
+              '--key_size 512 --control_location front-end')  # old style
+    def test_encryption_type_create_with_args(self, arg):
+        expected = {'encryption': {'cipher': None,
+                                   'key_size': 512,
+                                   'provider': 'TestProvider',
+                                   'control_location': 'front-end'}}
+        self.run_command('encryption-type-create 2 TestProvider ' + arg)
+        self.assert_called('POST', '/types/2/encryption', body=expected)
+        self.assert_called_anytime('GET', '/types/2')
+
     def test_encryption_type_update(self):
         """
         Test encryption-type-update shell command.
@@ -828,17 +995,44 @@ class ShellTest(utils.TestCase):
         self.assert_called('POST', '/volumes/1234/action', body=expected)
 
     def test_snapshot_delete(self):
+        """Tests delete snapshot without force parameter"""
         self.run_command('snapshot-delete 1234')
         self.assert_called('DELETE', '/snapshots/1234')
 
+    def test_snapshot_delete_multiple(self):
+        """Tests delete multiple snapshots without force parameter"""
+        self.run_command('snapshot-delete 5678 1234')
+        self.assert_called_anytime('DELETE', '/snapshots/5678')
+        self.assert_called('DELETE', '/snapshots/1234')
+
+    def test_force_snapshot_delete(self):
+        """Tests delete snapshot with default force parameter value(True)"""
+        self.run_command('snapshot-delete 1234 --force')
+        expected_body = {'os-force_delete': None}
+        self.assert_called('POST',
+                           '/snapshots/1234/action',
+                           expected_body)
+
+    def test_force_snapshot_delete_multiple(self):
+        """
+        Tests delete multiple snapshots with force parameter
+
+        Snapshot delete with force parameter allows deleting snapshot of a
+        volume when its status is other than "available" or "error".
+        """
+        self.run_command('snapshot-delete 5678 1234 --force')
+        expected_body = {'os-force_delete': None}
+        self.assert_called_anytime('POST',
+                                   '/snapshots/5678/action',
+                                   expected_body)
+        self.assert_called_anytime('POST',
+                                   '/snapshots/1234/action',
+                                   expected_body)
+
     def test_quota_delete(self):
         self.run_command('quota-delete 1234')
         self.assert_called('DELETE', '/os-quota-sets/1234')
 
-    def test_snapshot_delete_multiple(self):
-        self.run_command('snapshot-delete 5678')
-        self.assert_called('DELETE', '/snapshots/5678')
-
     def test_volume_manage(self):
         self.run_command('manage host1 some_fake_name '
                          '--name foo --description bar '
@@ -918,6 +1112,18 @@ class ShellTest(utils.TestCase):
                                'bootable': False}}
         self.assert_called_anytime('POST', '/os-volume-manage', body=expected)
 
+    def test_volume_manageable_list(self):
+        self.run_command('manageable-list fakehost')
+        self.assert_called('GET', '/os-volume-manage/detail?host=fakehost')
+
+    def test_volume_manageable_list_details(self):
+        self.run_command('manageable-list fakehost --detailed True')
+        self.assert_called('GET', '/os-volume-manage/detail?host=fakehost')
+
+    def test_volume_manageable_list_no_details(self):
+        self.run_command('manageable-list fakehost --detailed False')
+        self.assert_called('GET', '/os-volume-manage?host=fakehost')
+
     def test_volume_unmanage(self):
         self.run_command('unmanage 1234')
         self.assert_called('POST', '/volumes/1234/action',
@@ -989,8 +1195,8 @@ class ShellTest(utils.TestCase):
         self.assert_called('PUT', '/consistencygroups/1234',
                            body=expected)
 
-    def test_consistencygroup_update_bad_request(self):
-        self.assertRaises(exceptions.BadRequest,
+    def test_consistencygroup_update_invalid_args(self):
+        self.assertRaises(exceptions.ClientException,
                           self.run_command,
                           'consisgroup-update 1234')
 
@@ -1031,13 +1237,13 @@ class ShellTest(utils.TestCase):
                            expected)
 
     def test_consistencygroup_create_from_src_fail_no_snap_cg(self):
-        self.assertRaises(exceptions.BadRequest,
+        self.assertRaises(exceptions.ClientException,
                           self.run_command,
                           'consisgroup-create-from-src '
                           '--name cg')
 
     def test_consistencygroup_create_from_src_fail_both_snap_cg(self):
-        self.assertRaises(exceptions.BadRequest,
+        self.assertRaises(exceptions.ClientException,
                           self.run_command,
                           'consisgroup-create-from-src '
                           '--name cg '
@@ -1088,6 +1294,102 @@ class ShellTest(utils.TestCase):
         self.run_command('backup-list')
         self.assert_called('GET', '/backups/detail')
 
+    @mock.patch("cinderclient.utils.print_list")
+    def test_backup_list_sort(self, mock_print_list):
+        self.run_command('backup-list --sort id')
+        self.assert_called('GET', '/backups/detail?sort=id')
+        columns = ['ID', 'Volume ID', 'Status', 'Name', 'Size', 'Object Count',
+               'Container']
+        mock_print_list.assert_called_once_with(mock.ANY, columns,
+            sortby_index=None)
+
+    def test_backup_list_data_timestamp(self):
+        self.run_command('backup-list --sort data_timestamp')
+        self.assert_called('GET', '/backups/detail?sort=data_timestamp')
+
     def test_get_capabilities(self):
         self.run_command('get-capabilities host')
         self.assert_called('GET', '/capabilities/host')
+
+    def test_image_metadata_show(self):
+        # since the request is not actually sent to cinder API but is
+        # calling the method in :class:`v2.fakes.FakeHTTPClient` instead.
+        # Thus, ignore any exception which is false negative compare
+        # with real API call.
+        try:
+            self.run_command('image-metadata-show 1234')
+        except Exception:
+            pass
+        expected = {"os-show_image_metadata": None}
+        self.assert_called('POST', '/volumes/1234/action', body=expected)
+
+    def test_snapshot_manage(self):
+        self.run_command('snapshot-manage 1234 some_fake_name '
+                         '--name foo --description bar '
+                         '--metadata k1=v1 k2=v2')
+        expected = {'snapshot': {'volume_id': 1234,
+                                 'ref': {'source-name': 'some_fake_name'},
+                                 'name': 'foo',
+                                 'description': 'bar',
+                                 'metadata': {'k1': 'v1', 'k2': 'v2'}
+                                 }}
+        self.assert_called_anytime('POST', '/os-snapshot-manage',
+                                   body=expected)
+
+    def test_snapshot_manageable_list(self):
+        self.run_command('snapshot-manageable-list fakehost')
+        self.assert_called('GET', '/os-snapshot-manage/detail?host=fakehost')
+
+    def test_snapshot_manageable_list_details(self):
+        self.run_command('snapshot-manageable-list fakehost --detailed True')
+        self.assert_called('GET', '/os-snapshot-manage/detail?host=fakehost')
+
+    def test_snapshot_manageable_list_no_details(self):
+        self.run_command('snapshot-manageable-list fakehost --detailed False')
+        self.assert_called('GET', '/os-snapshot-manage?host=fakehost')
+
+    def test_snapshot_unmanage(self):
+        self.run_command('snapshot-unmanage 1234')
+        self.assert_called('POST', '/snapshots/1234/action',
+                           body={'os-unmanage': None})
+
+    def test_extra_specs_list(self):
+        self.run_command('extra-specs-list')
+        self.assert_called('GET', '/types?is_public=None')
+
+    def test_quota_class_show(self):
+        self.run_command('quota-class-show test')
+        self.assert_called('GET', '/os-quota-class-sets/test')
+
+    def test_quota_class_update(self):
+        expected = {'quota_class_set': {'volumes': 2,
+                                        'snapshots': 2,
+                                        'gigabytes': 1,
+                                        'backups': 1,
+                                        'backup_gigabytes': 1,
+                                        'per_volume_gigabytes': 1}}
+        self.run_command('quota-class-update test '
+                         '--volumes 2 '
+                         '--snapshots 2 '
+                         '--gigabytes 1 '
+                         '--backups 1 '
+                         '--backup-gigabytes 1 '
+                         '--per-volume-gigabytes 1')
+        self.assert_called('PUT', '/os-quota-class-sets/test', body=expected)
+
+    def test_translate_attachments(self):
+        attachment_id = 'aaaa'
+        server_id = 'bbbb'
+        obj_id = 'cccc'
+        info = {
+            'attachments': [{
+                'attachment_id': attachment_id,
+                'id': obj_id,
+                'server_id': server_id}]
+            }
+
+        new_info = test_shell._translate_attachments(info)
+
+        self.assertEqual(attachment_id, new_info['attachment_ids'][0])
+        self.assertEqual(server_id, new_info['attached_servers'][0])
+        self.assertNotIn('id', new_info)
