@@ -14,10 +14,11 @@
 #    under the License.
 
 """
-Volume Backups interface (1.1 extension).
+Volume Backups interface (v2 extension).
 """
+
+from cinderclient.apiclient import base as common_base
 from cinderclient import base
-from cinderclient.openstack.common.apiclient import base as common_base
 
 
 class VolumeBackup(base.Resource):
@@ -26,13 +27,17 @@ class VolumeBackup(base.Resource):
     def __repr__(self):
         return "<VolumeBackup: %s>" % self.id
 
-    def delete(self):
+    def delete(self, force=False):
         """Delete this volume backup."""
-        return self.manager.delete(self)
+        return self.manager.delete(self, force)
 
     def reset_state(self, state):
         return self.manager.reset_state(self, state)
 
+    def update(self, **kwargs):
+        """Update the name or description for this backup."""
+        return self.manager.update(self, **kwargs)
+
 
 class VolumeBackupManager(base.ManagerWithFind):
     """Manage :class:`VolumeBackup` resources."""
@@ -50,6 +55,9 @@ class VolumeBackupManager(base.ManagerWithFind):
         :param description: The description of the backup.
         :param incremental: Incremental backup.
         :param force: If True, allows an in-use volume to be backed up.
+        :param snapshot_id: The ID of the snapshot to backup. This should
+                            be a snapshot of the src volume, when specified,
+                            the new backup will be based on the snapshot.
         :rtype: :class:`VolumeBackup`
         """
         body = {'backup': {'volume_id': volume_id,
@@ -81,16 +89,21 @@ class VolumeBackupManager(base.ManagerWithFind):
                                    limit=limit, sort=sort)
         return self._list(url, resource_type, limit=limit)
 
-    def delete(self, backup):
+    def delete(self, backup, force=False):
         """Delete a volume backup.
 
         :param backup: The :class:`VolumeBackup` to delete.
+        :param force: Allow delete in state other than error or available.
         """
-        return self._delete("/backups/%s" % base.getid(backup))
+        if force:
+            return self._action('os-force_delete', backup)
+        else:
+            return self._delete("/backups/%s" % base.getid(backup))
 
     def reset_state(self, backup, state):
         """Update the specified volume backup with the provided state."""
-        return self._action('os-reset_status', backup, {'status': state})
+        return self._action('os-reset_status', backup,
+                            {'status': state} if state else {})
 
     def _action(self, action, backup, info=None, **kwargs):
         """Perform a volume backup action."""
